# Script for plotting the average Nusselt number during a given 
# time period at the end of the model run from on the statistics file
# (for different Rayleigh numbers).

import numpy as np
import pandas as pd
import os
import matplotlib.pyplot as plt
from matplotlib.ticker import FormatStrFormatter

Rayleigh_numbers = ["10000", "100000", "400000", "2000000"]
Phase_buoyancy = ["0.4", "0.2", "0.0", "-0.1", "-0.2", "-0.3", "-0.4", "-0.5", "-0.6", "-0.7", "-0.8", "-0.9"]
colors = ["#EFF000", "#7DBD3B", "#4A867F", "#2C3791"]

fig1, ax1 = plt.subplots(figsize=(8,10), dpi=150)

for Ra, c in zip(Rayleigh_numbers, colors):
	Nusselt_array = np.empty((0,2), float)
	steady = np.zeros(0, dtype=bool)
	
	for P in Phase_buoyancy:
		
		filename="./Ra" + Ra + "_P" + P + "_d0.05_refinement7/statistics"

		header = []
		header_read = True

		# In case the model output hasn't been generated yet and the file doesn't exist, 
		# skip that parameter combination
		try:
			my_file = open(filename)
			with open(filename) as f:
				while header_read:
					line = f.readline()
					if line[0] == '#':
						idx_start = line.find(":")
						header.append(line[idx_start+2:-1])
					else:
						header_read = False

			values = pd.read_csv(filename, skiprows=len(header), header=None, delim_whitespace=True, names=header)

			# Average the heat flux over a fixed time at the end of the model run
			last_line = values.tail(1)
			averaging_duration = 1.e7
			if last_line.iloc[0]["Time (years)"] > 1.99e8:                  #model has reached end time
				averaging_duration = 1.e8
				steady = np.append(steady, False)
			else:
				steady = np.append(steady, True)
			steady_state_time = last_line.iloc[0]["Time (years)"] - averaging_duration
			index = values["Time (years)"].gt(steady_state_time).idxmax()
			mean = values[index:]["Outward heat flux through boundary with indicator 3 (\"top\") (W)"].mean()
		
			# To compute the Nusselt number, we need to scale the heat flux with the thermal conductivity 
			# and the density. The value of the thermal conductivity comes from the ASPECT input files, 
			# where it is used to set the Rayleight number as Ra = rho^2 * g * cp * alpha * DeltaT * h^3 / (eta * k), 
			# with
			# density rho = 1000 kg/m^3
			# gravity g = 10 m/s^2
			# thermal expansivity alpha = 1e-4 1/K
			# temperature difference between top and bottom DeltaT = 1000 K
			# size of the model domain h = 1350 km
			# viscosity eta = 1e20 Pa s
			# specific heat cp = 1000 J/kg/K 
			thermal_conductivity = 24603750.0/float(Ra)
			density = 1000.0
			scale = thermal_conductivity*density
			Nusselt_array = np.append(Nusselt_array, np.array([[float(P),float(mean)/scale]]), axis=0)

		except IOError:
    			# file didn't exist (or other issues)
    			pass
    			
	ax1.plot(Nusselt_array[:,0], Nusselt_array[:,1], label="Ra = " + Ra, color=c, marker='o', fillstyle='none')
	ax1.plot(Nusselt_array[steady,0], Nusselt_array[steady,1], label='_nolegend_', color=c, marker='o', fillstyle='full', linestyle='None')

fig = plt.gcf()

# Legend
leg1 = ax1.legend(loc='best',fontsize=14)
leg1.get_frame().set_edgecolor('k')
leg1.get_frame().set_linewidth(1.5)

# Axes
ax1.set_xlabel("Phase Buoyancy number"    , fontsize=15, labelpad=15)
ax1.set_ylabel("Nusselt number", fontsize=15, labelpad=15)
ax1.set_yscale('log')
ax1.set_xlim([-0.8, 0.5])
ax1.set_ylim([1, 50])

# Format ticks and tick labels
plt.tick_params(axis='y', which='minor')
ax1.yaxis.set_minor_formatter(FormatStrFormatter("%.0f"))
ax1.yaxis.set_major_formatter(FormatStrFormatter("%.0f"))

plt.setp(ax1.get_yticklabels(which='both'), fontsize=14)
plt.setp(ax1.get_xticklabels(), fontsize=14)
ax1.tick_params('both', length=8, width=1.5, which='major')
ax1.tick_params('both', length=5, width=1.5, which='minor')

# Format the grid
for axis in ['top','bottom','left','right']:
  ax1.spines[axis].set_linewidth(1.5)
ax1.grid(which='minor', alpha=0.2)
ax1.grid(which='major', alpha=0.7)

# prevents axis labels being cut off
plt.tight_layout()

# Save the plot
plt.savefig('heat-flux-Ra.png')
